#ifndef DUNE_TECTONIC_GLOBAL_RUINA_NONLINEARITY_HH
#define DUNE_TECTONIC_GLOBAL_RUINA_NONLINEARITY_HH

#include <vector>

#include <dune/common/fmatrix.hh>
#include <dune/common/fvector.hh>
#include <dune/istl/bcrsmatrix.hh>
#include <dune/istl/bvector.hh>

#include "globalnonlinearity.hh"
#include "localnonlinearity.hh"
#include "nicefunction.hh"

namespace Dune {
template <int dim>
class GlobalRuinaNonlinearity : public GlobalNonlinearity<dim> {
public:
  GlobalRuinaNonlinearity(
      shared_ptr<BlockVector<FieldVector<double, 1>> const> nodalIntegrals,
      shared_ptr<BlockVector<FieldVector<double, 1>> const> a,
      shared_ptr<BlockVector<FieldVector<double, 1>> const> mu,
      shared_ptr<BlockVector<FieldVector<double, 1>> const> eta,
      shared_ptr<BlockVector<FieldVector<double, 1>> const> normalStress)
      : nodalIntegrals(nodalIntegrals),
        a(a),
        mu(mu),
        eta(eta),
        normalStress(normalStress),
        trivialNonlinearity(new LocalNonlinearity<dim>(
            shared_ptr<NiceFunction const>(new TrivialFunction))) {}

  /*
    Return a restriction of the outer function to the i'th node.
  */
  virtual shared_ptr<LocalNonlinearity<dim> const> restriction(int i) const {
    if ((*nodalIntegrals)[i][0] == 0)
      return trivialNonlinearity;

    shared_ptr<NiceFunction const> const func(
        new RuinaFunction((*nodalIntegrals)[i][0], (*a)[i][0], (*mu)[i][0],
                          (*eta)[i][0], (*normalStress)[i][0]));
    return shared_ptr<LocalNonlinearity<dim>>(new LocalNonlinearity<dim>(func));
  }

private:
  shared_ptr<LocalNonlinearity<dim> const> const trivialNonlinearity;

  shared_ptr<BlockVector<FieldVector<double, 1>> const> nodalIntegrals;
  shared_ptr<BlockVector<FieldVector<double, 1>> const> a;
  shared_ptr<BlockVector<FieldVector<double, 1>> const> mu;
  shared_ptr<BlockVector<FieldVector<double, 1>> const> eta;
  shared_ptr<BlockVector<FieldVector<double, 1>> const> normalStress;
};
}
#endif
