#include <octave/oct.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <dune/tectonic/octave/duneoctave.hh>
#include <dune/tectonic/samplefunctional.hh>

#include <cassert>

DEFUN_DLD(duneevaluate, args, nargout, "duneevaluate(A,b,points)\n\
\n\
Evaluate x -> 1/2<Ax,x> - <b,x> + H(|x|) at each point y that is a column vector of A using DUNE\n") {
  int const dim = 2;

  assert(args.length() == 4);
  assert(nargout <= 1);

  typedef Dune::SampleFunctional<dim> Functional;

  Functional::SmallMatrix A;
  Dune::octaveToDune<dim>(args(0).matrix_value(), A);

  Functional::SmallVector b;
  Dune::octaveToDune<dim>(args(1).vector_value(), b);

  Dune::SampleFunction<100> f_steep;
  Dune::SampleFunction<2> f_sample;

  charNDArray bar = args(2).char_array_value();
  Dune::NiceFunction *foo;
  switch (bar(0)) {
    case 'S':
      foo = &f_steep;
      break;
    case 's':
      foo = &f_sample;
      break;
    default:
      assert(false);
  }
  Dune::LocalNonlinearity<dim> phi(*foo);
  Functional J(A, b, phi);

  Matrix points(args(3).matrix_value());
  size_t const columns = points.columns();
  RowVector ret(columns);
  Functional::SmallVector our_point;
  for (size_t i = 0; i < columns; ++i) {
    Dune::octaveToDune<dim>(points.column(i), our_point);
    ret(i) = J(our_point);
  }
  return octave_value(ret);
}
