#include <octave/oct.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <dune/tectonic/octave/duneoctave.hh>
#include <dune/tectonic/samplefunctional.hh>

#include <cassert>

DEFUN_DLD(duneminimise, args, nargout, "duneminimise(A,b,f,y)\n\
\n\
Make a minimisation step of x -> 1/2<Ax,x> - <b,x> + H(|x|) using DUNE starting at x\n") {
  int const dim = 2;

  assert(args.length() == 4);
  assert(nargout <= 1);

  typedef Dune::SampleFunctional<dim> Functional;

  Functional::SmallMatrix A;
  Dune::octaveToDune<dim>(args(0).matrix_value(), A);

  Functional::SmallVector b;
  Dune::octaveToDune<dim>(args(1).vector_value(), b);

  Dune::SampleFunction<100> f_steep;
  Dune::SampleFunction<2> f_sample;

  charNDArray bar = args(2).char_array_value();
  Dune::NiceFunction *foo;
  switch (bar(0)) {
    case 'S':
      foo = &f_steep;
      break;
    case 's':
      foo = &f_sample;
      break;
    default:
      assert(false);
  }
  Dune::LocalNonlinearity<dim> phi(*foo);
  Functional J(A, b, phi);

  ColumnVector start_octave(args(3).vector_value());
  Functional::SmallVector start;
  Dune::octaveToDune<dim>(start_octave, start);

  Dune::minimise(J, start);

  Dune::duneToOctave<dim>(start, start_octave);
  return octave_value(start_octave);
}
